@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2010, 2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,19 +28,24 @@
 #include <string.h>
 #include <math.h>
 #include <sys/types.h>
-#include <pthread.h>
 #include <signal.h>
 
 #include <starpu.h>
 
-#ifdef STARPU_USE_GORDON
-#include <gordon/null.h>
+#ifdef STARPU_HAVE_HELGRIND_H
+#include <valgrind/helgrind.h>
+#endif
+#ifndef ANNOTATE_HAPPENS_BEFORE
+#define ANNOTATE_HAPPENS_BEFORE(obj) ((void)0)
+#endif
+#ifndef ANNOTATE_HAPPENS_AFTER
+#define ANNOTATE_HAPPENS_AFTER(obj) ((void)0)
 #endif
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 #define TAG(i, j, iter)	((starpu_tag_t) ( ((uint64_t)(iter)<<48) |  ((uint64_t)(j)<<24) | (i)) )
 
-struct starpu_codelet cl = {};
+struct starpu_codelet cl;
 
 #ifdef STARPU_QUICK_CHECK
 #define Ni	32
@@ -90,25 +94,25 @@ static void parse_args(int argc, char **argv)
 void callback_cpu(void *argcb);
 static void express_deps(unsigned i, unsigned j, unsigned iter);
 
-static void tag_cleanup_grid(unsigned ni, unsigned nj, unsigned iter)
+static void tag_cleanup_grid(unsigned piter)
 {
 	unsigned i,j;
 
 	for (j = 0; j < nj; j++)
 	for (i = 0; i < ni; i++)
 	{
-		starpu_tag_remove(TAG(i,j,iter));
+		starpu_tag_remove(TAG(i,j,piter));
 	}
 
 
-} 
+}
 
-static int create_task_grid(unsigned iter)
+static int create_task_grid(unsigned piter)
 {
 	unsigned i, j;
 	int ret;
 
-/*	FPRINTF(stderr, "start iter %d...\n", iter); */
+/*	FPRINTF(stderr, "start iter %d...\n", piter); */
 	callback_cnt = (ni*nj);
 
 	/* create non-entry tasks */
@@ -123,10 +127,10 @@ static int create_task_grid(unsigned iter)
 		task->cl_arg = NULL;
 
 		task->use_tag = 1;
-		task->tag_id = TAG(i, j, iter);
+		task->tag_id = TAG(i, j, piter);
 
 		/* express deps : (i,j) depends on (i-1, j-1) & (i-1, j+1) */
-		express_deps(i, j, iter);
+		express_deps(i, j, piter);
 
 		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) return 77;
@@ -144,7 +148,7 @@ static int create_task_grid(unsigned iter)
 
 		task->use_tag = 1;
 		/* this is an entry task */
-		task->tag_id = TAG(0, j, iter);
+		task->tag_id = TAG(0, j, piter);
 
 		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) return 77;
@@ -154,17 +158,20 @@ static int create_task_grid(unsigned iter)
 }
 
 
-void callback_cpu(void *argcb __attribute__ ((unused)))
+void callback_cpu(void *argcb)
 {
-	unsigned newcnt = STARPU_ATOMIC_ADD(&callback_cnt, -1);	
+	(void)argcb;
+	unsigned newcnt = STARPU_ATOMIC_ADD(&callback_cnt, -1);
+	ANNOTATE_HAPPENS_BEFORE(&callback_cnt);
 
 	if (newcnt == 0)
 	{
+		ANNOTATE_HAPPENS_AFTER(&callback_cnt);
 		if (++iter < nk)
 		{
 			/* cleanup old grids ... */
 			if (iter > 2)
-				tag_cleanup_grid(ni, nj, iter-2);
+				tag_cleanup_grid(iter-2);
 
 			/* create a new iteration */
 			create_task_grid(iter);
@@ -172,13 +179,14 @@ void callback_cpu(void *argcb __attribute__ ((unused)))
 	}
 }
 
-void cpu_codelet(void *descr[] __attribute__((unused)),
-			void *_args __attribute__ ((unused)))
+void cpu_codelet(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 /*	printf("execute task\n"); */
 }
 
-static void express_deps(unsigned i, unsigned j, unsigned iter)
+static void express_deps(unsigned i, unsigned j, unsigned piter)
 {
 	if (j > 0)
 	{
@@ -186,12 +194,12 @@ static void express_deps(unsigned i, unsigned j, unsigned iter)
 		if (j < nj - 1)
 		{
 			/* (i,j+1) exists */
-			starpu_tag_declare_deps(TAG(i,j,iter), 2, TAG(i-1,j-1,iter), TAG(i-1,j+1,iter));
+			starpu_tag_declare_deps(TAG(i,j,piter), 2, TAG(i-1,j-1,piter), TAG(i-1,j+1,piter));
 		}
 		else
 		{
 			/* (i,j+1) does not exist */
-			starpu_tag_declare_deps(TAG(i,j,iter), 1, TAG(i-1,j-1,iter));
+			starpu_tag_declare_deps(TAG(i,j,piter), 1, TAG(i-1,j-1,piter));
 		}
 	}
 	else
@@ -200,7 +208,7 @@ static void express_deps(unsigned i, unsigned j, unsigned iter)
 		if (j < nj - 1)
 		{
 			/* (i,j+1) exists */
-			starpu_tag_declare_deps(TAG(i,j,iter), 1, TAG(i-1,j+1,iter));
+			starpu_tag_declare_deps(TAG(i,j,piter), 1, TAG(i-1,j+1,piter));
 		}
 		else
 		{
@@ -210,38 +218,41 @@ static void express_deps(unsigned i, unsigned j, unsigned iter)
 	}
 }
 
-int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
+int main(int argc, char **argv)
 {
 	int ret;
 
+#ifdef STARPU_HAVE_HELGRIND_H
+	if (RUNNING_ON_VALGRIND) {
+		ni /= 2;
+		nj /= 2;
+		nk /= 2;
+	}
+#endif
+
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV)
 		exit(77);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-#ifdef STARPU_USE_GORDON
-	/* load an empty kernel and get its identifier */
-	unsigned gordon_null_kernel = load_gordon_null_kernel();
-#endif
-
 	parse_args(argc, argv);
 
 	FPRINTF(stderr, "ITER: %u\n", nk);
 
+	starpu_codelet_init(&cl);
 	cl.cpu_funcs[0] = cpu_codelet;
+	cl.cpu_funcs_name[0] = "cpu_codelet";
 	cl.cuda_funcs[0] = cpu_codelet;
 	cl.opencl_funcs[0] = cpu_codelet;
-#ifdef STARPU_USE_GORDON
-	cl.gordon_func = gordon_null_kernel;
-#endif
 	cl.nbuffers = 0;
+	cl.name = "dummy";
 
 	ret = create_task_grid(0);
 	if (ret == 0)
 	     starpu_task_wait_for_all();
 
-	tag_cleanup_grid(ni, nj, nk-2);
-	tag_cleanup_grid(ni, nj, nk-1);
+	tag_cleanup_grid(nk-2);
+	tag_cleanup_grid(nk-1);
 
 	starpu_shutdown();
 
