@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013,2015-2017                      CNRS
- * Copyright (C) 2009-2012,2014,2015,2018                 Université de Bordeaux
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2010                                     Mehdi Juhoor
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -57,19 +55,21 @@ int main(int argc, char **argv)
 
 	dw_cholesky(bmat, size/nblocks, rank, nodes, &timing, &flops);
 
-	starpu_cublas_shutdown();
-	starpu_mpi_shutdown();
-
 #ifndef STARPU_SIMGRID
 	matrix_display(bmat, rank);
 
-	dw_cholesky_check_computation(bmat, rank, nodes, &correctness, &flops);
+	if (check)
+		dw_cholesky_check_computation(bmat, rank, nodes, &correctness, &flops, 0.001);
 #endif
 
 	matrix_free(&bmat, rank, nodes, 1);
 
+	starpu_cublas_shutdown();
+	starpu_mpi_shutdown();
+
 #ifndef STARPU_SIMGRID
-	assert(correctness);
+	if (check)
+		assert(correctness);
 #endif
 
 	if (rank == 0)
