@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2016  CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Joris Pablo
+ * Copyright (C) 2018,2020  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,8 +20,10 @@
 #define __FXT_H__
 
 
+/** @file */
+
 #ifndef _GNU_SOURCE
-#define _GNU_SOURCE  /* ou _BSD_SOURCE ou _SVID_SOURCE */
+#define _GNU_SOURCE  1 /* ou _BSD_SOURCE ou _SVID_SOURCE */
 #endif
 
 #include <string.h>
@@ -39,7 +42,7 @@
 #define _STARPU_FUT_CUDA_KEY	0x102
 #define _STARPU_FUT_OPENCL_KEY	0x103
 #define _STARPU_FUT_MIC_KEY	0x104
-#define _STARPU_FUT_SCC_KEY	0x105
+#define _STARPU_FUT_MPI_KEY	0x106
 
 #define _STARPU_FUT_WORKER_INIT_START	0x5100
 #define _STARPU_FUT_WORKER_INIT_END	0x5101
@@ -52,10 +55,10 @@
 
 #define _STARPU_FUT_UPDATE_TASK_CNT	0x5106
 
-#define _STARPU_FUT_START_FETCH_INPUT	0x5107
-#define _STARPU_FUT_END_FETCH_INPUT	0x5108
-#define _STARPU_FUT_START_PUSH_OUTPUT	0x5109
-#define _STARPU_FUT_END_PUSH_OUTPUT	0x5110
+#define _STARPU_FUT_START_FETCH_INPUT_ON_TID	0x5107
+#define _STARPU_FUT_END_FETCH_INPUT_ON_TID	0x5108
+#define _STARPU_FUT_START_PUSH_OUTPUT_ON_TID	0x5109
+#define _STARPU_FUT_END_PUSH_OUTPUT_ON_TID	0x5110
 
 #define _STARPU_FUT_TAG		0x5111
 #define _STARPU_FUT_TAG_DEPS	0x5112
@@ -83,10 +86,10 @@
 #define _STARPU_FUT_USER_DEFINED_START	0x5120
 #define _STARPU_FUT_USER_DEFINED_END	0x5121
 
-#define	_STARPU_FUT_NEW_MEM_NODE		0x5122
+#define	_STARPU_FUT_NEW_MEM_NODE	0x5122
 
 #define	_STARPU_FUT_START_CALLBACK	0x5123
-#define	_STARPU_FUT_END_CALLBACK		0x5124
+#define	_STARPU_FUT_END_CALLBACK	0x5124
 
 #define	_STARPU_FUT_TASK_DONE		0x5125
 #define	_STARPU_FUT_TAG_DONE		0x5126
@@ -101,6 +104,12 @@
 
 #define _STARPU_FUT_TASK_NAME	0x512b
 
+#define _STARPU_FUT_DATA_WONT_USE	0x512c
+
+#define _STARPU_FUT_TASK_COLOR	0x512d
+
+#define _STARPU_FUT_DATA_DOING_WONT_USE	0x512e
+
 #define	_STARPU_FUT_START_MEMRECLAIM	0x5131
 #define	_STARPU_FUT_END_MEMRECLAIM	0x5132
 
@@ -119,46 +128,46 @@
 
 #define _STARPU_FUT_TASK_WAIT_FOR_ALL	0x513b
 
-#define _STARPU_FUT_EVENT	0x513c
+#define _STARPU_FUT_EVENT		0x513c
 #define _STARPU_FUT_THREAD_EVENT	0x513d
 
 #define	_STARPU_FUT_CODELET_DETAILS	0x513e
 #define	_STARPU_FUT_CODELET_DATA	0x513f
 
-#define _STARPU_FUT_LOCKING_MUTEX	0x5140	
-#define _STARPU_FUT_MUTEX_LOCKED	0x5141	
+#define _STARPU_FUT_LOCKING_MUTEX	0x5140
+#define _STARPU_FUT_MUTEX_LOCKED	0x5141
 
-#define _STARPU_FUT_UNLOCKING_MUTEX		0x5142	
-#define _STARPU_FUT_MUTEX_UNLOCKED		0x5143	
+#define _STARPU_FUT_UNLOCKING_MUTEX	0x5142
+#define _STARPU_FUT_MUTEX_UNLOCKED	0x5143
 
-#define _STARPU_FUT_TRYLOCK_MUTEX		0x5144	
+#define _STARPU_FUT_TRYLOCK_MUTEX	0x5144
 
-#define _STARPU_FUT_RDLOCKING_RWLOCK	0x5145	
-#define _STARPU_FUT_RWLOCK_RDLOCKED		0x5146	
+#define _STARPU_FUT_RDLOCKING_RWLOCK	0x5145
+#define _STARPU_FUT_RWLOCK_RDLOCKED	0x5146
 
-#define _STARPU_FUT_WRLOCKING_RWLOCK	0x5147	
-#define _STARPU_FUT_RWLOCK_WRLOCKED		0x5148	
+#define _STARPU_FUT_WRLOCKING_RWLOCK	0x5147
+#define _STARPU_FUT_RWLOCK_WRLOCKED	0x5148
 
-#define _STARPU_FUT_UNLOCKING_RWLOCK	0x5149	
-#define _STARPU_FUT_RWLOCK_UNLOCKED		0x514a	
+#define _STARPU_FUT_UNLOCKING_RWLOCK	0x5149
+#define _STARPU_FUT_RWLOCK_UNLOCKED	0x514a
 
-#define _STARPU_FUT_LOCKING_SPINLOCK	0x514b	
-#define _STARPU_FUT_SPINLOCK_LOCKED		0x514c	
+#define _STARPU_FUT_LOCKING_SPINLOCK	0x514b
+#define _STARPU_FUT_SPINLOCK_LOCKED	0x514c
 
-#define _STARPU_FUT_UNLOCKING_SPINLOCK		0x514d	
-#define _STARPU_FUT_SPINLOCK_UNLOCKED		0x514e	
+#define _STARPU_FUT_UNLOCKING_SPINLOCK	0x514d
+#define _STARPU_FUT_SPINLOCK_UNLOCKED	0x514e
 
-#define _STARPU_FUT_TRYLOCK_SPINLOCK		0x514f	
+#define _STARPU_FUT_TRYLOCK_SPINLOCK	0x514f
 
-#define _STARPU_FUT_COND_WAIT_BEGIN		0x5150
-#define _STARPU_FUT_COND_WAIT_END		0x5151
+#define _STARPU_FUT_COND_WAIT_BEGIN	0x5150
+#define _STARPU_FUT_COND_WAIT_END	0x5151
 
-#define _STARPU_FUT_MEMORY_FULL			0x5152
+#define _STARPU_FUT_MEMORY_FULL		0x5152
 
-#define _STARPU_FUT_DATA_LOAD 0x5153
+#define _STARPU_FUT_DATA_LOAD 		0x5153
 
-#define _STARPU_FUT_START_UNPARTITION 0x5154
-#define _STARPU_FUT_END_UNPARTITION 0x5155
+#define _STARPU_FUT_START_UNPARTITION_ON_TID 0x5154
+#define _STARPU_FUT_END_UNPARTITION_ON_TID 0x5155
 
 #define	_STARPU_FUT_START_FREE		0x5156
 #define	_STARPU_FUT_END_FREE		0x5157
@@ -170,7 +179,7 @@
 #define _STARPU_FUT_SCHED_COMPONENT_POP_PRIO 	0x515b
 
 #define	_STARPU_FUT_START_WRITEBACK_ASYNC	0x515c
-#define	_STARPU_FUT_END_WRITEBACK_ASYNC	0x515d
+#define	_STARPU_FUT_END_WRITEBACK_ASYNC		0x515d
 
 #define	_STARPU_FUT_HYPERVISOR_BEGIN    0x5160
 #define	_STARPU_FUT_HYPERVISOR_END	0x5161
@@ -198,7 +207,7 @@
 #define _STARPU_FUT_TASK_BUILD_END	0x5171
 
 #define _STARPU_FUT_TASK_MPI_DECODE_START	0x5172
-#define _STARPU_FUT_TASK_MPI_DECODE_END	0x5173
+#define _STARPU_FUT_TASK_MPI_DECODE_END		0x5173
 
 #define _STARPU_FUT_TASK_MPI_PRE_START	0x5174
 #define _STARPU_FUT_TASK_MPI_PRE_END	0x5175
@@ -212,12 +221,29 @@
 #define _STARPU_FUT_TASK_WAIT_FOR_ALL_START	0x517a
 #define _STARPU_FUT_TASK_WAIT_FOR_ALL_END	0x517b
 
-#define _STARPU_FUT_HANDLE_DATA_REGISTER 0x517c
-#define _STARPU_FUT_DATA_INVALIDATE 0x517d
+#define _STARPU_FUT_HANDLE_DATA_REGISTER	0x517c
+
+#define _STARPU_FUT_START_FETCH_INPUT	0x517e
+#define _STARPU_FUT_END_FETCH_INPUT	0x517f
 
 #define _STARPU_FUT_TASK_THROTTLE_START	0x5180
 #define _STARPU_FUT_TASK_THROTTLE_END	0x5181
 
+#define _STARPU_FUT_DATA_STATE_INVALID 0x5182
+#define _STARPU_FUT_DATA_STATE_OWNER      0x5183
+#define _STARPU_FUT_DATA_STATE_SHARED     0x5184
+
+#define _STARPU_FUT_DATA_REQUEST_CREATED   0x5185
+
+extern unsigned long _starpu_job_cnt;
+
+static inline unsigned long _starpu_fxt_get_job_id(void)
+{
+	unsigned long ret = STARPU_ATOMIC_ADDL(&_starpu_job_cnt, 1);
+	STARPU_ASSERT_MSG(_starpu_job_cnt != 0, "Oops, job_id wrapped! There are too many tasks for tracking them for profiling");
+	return ret;
+}
+
 #ifdef STARPU_USE_FXT
 #include <fxt/fxt.h>
 #include <fxt/fut.h>
@@ -235,17 +261,38 @@ void fut_set_filename(char *filename);
 #endif
 
 extern int _starpu_fxt_started;
+extern int _starpu_fxt_willstart;
 extern starpu_pthread_mutex_t _starpu_fxt_started_mutex;
 extern starpu_pthread_cond_t _starpu_fxt_started_cond;
 
+static inline void _starpu_fxt_wait_initialisation()
+{
+	STARPU_PTHREAD_MUTEX_LOCK(&_starpu_fxt_started_mutex);
+	while (_starpu_fxt_willstart && !_starpu_fxt_started)
+		STARPU_PTHREAD_COND_WAIT(&_starpu_fxt_started_cond, &_starpu_fxt_started_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&_starpu_fxt_started_mutex);
+}
+
+extern unsigned long _starpu_submit_order;
+
+static inline unsigned long _starpu_fxt_get_submit_order(void)
+{
+	unsigned long ret = STARPU_ATOMIC_ADDL(&_starpu_submit_order, 1);
+	STARPU_ASSERT_MSG(_starpu_submit_order != 0, "Oops, submit_order wrapped! There are too many tasks for tracking them for profiling");
+	return ret;
+}
+
 long _starpu_gettid(void);
 
-/* Initialize the FxT library. */
+/** Initialize the FxT library. */
 void _starpu_fxt_init_profiling(unsigned trace_buffer_size);
 
-/* Stop the FxT library, and generate the trace file. */
+/** Stop the FxT library, and generate the trace file. */
 void _starpu_stop_fxt_profiling(void);
 
+/** Generate the trace file. Used when catching signals SIGINT and SIGSEGV */
+void _starpu_fxt_dump_file(void);
+
 /* Associate the current processing unit to the identifier of the LWP that runs
  * the worker. */
 void _starpu_fxt_register_thread(unsigned);
@@ -259,7 +306,7 @@ void _starpu_fxt_register_thread(unsigned);
 #ifdef FUT_DO_PROBE1STR
 #define _STARPU_FUT_DO_PROBE1STR(CODE, P1, str) FUT_DO_PROBE1STR(CODE, P1, str)
 #else
-/* Sometimes we need something a little more specific than the wrappers from
+/** Sometimes we need something a little more specific than the wrappers from
  * FxT: these macro permit to put add an event with 3 (or 4) numbers followed
  * by a string. */
 #define _STARPU_FUT_DO_PROBE1STR(CODE, P1, str)			\
@@ -284,7 +331,7 @@ do {									\
 #ifdef FUT_DO_PROBE2STR
 #define _STARPU_FUT_DO_PROBE2STR(CODE, P1, P2, str) FUT_DO_PROBE2STR(CODE, P1, P2, str)
 #else
-/* Sometimes we need something a little more specific than the wrappers from
+/** Sometimes we need something a little more specific than the wrappers from
  * FxT: these macro permit to put add an event with 3 (or 4) numbers followed
  * by a string. */
 #define _STARPU_FUT_DO_PROBE2STR(CODE, P1, P2, str)			\
@@ -554,7 +601,7 @@ do {									\
 	{								\
 		/* we include the task name */			\
 		_STARPU_FUT_DO_PROBE5STR(_STARPU_FUT_START_CODELET_BODY, (job)->job_id, ((job)->task)->sched_ctx, workerid, starpu_worker_get_memory_node(workerid), 1, name); \
-		if (model_name && strcmp(model_name, name))				\
+		if (model_name)						\
 			_STARPU_FUT_DO_PROBE1STR(_STARPU_FUT_MODEL_NAME, (job)->job_id, model_name); \
 	}								\
 	else {                                                          \
@@ -580,7 +627,7 @@ do {									\
 		}							\
 		const size_t __job_size = _starpu_job_get_data_size((job)->task->cl?(job)->task->cl->model:NULL, perf_arch, nimpl, (job));	\
 		const uint32_t __job_hash = _starpu_compute_buffers_footprint((job)->task->cl?(job)->task->cl->model:NULL, perf_arch, nimpl, (job));\
-		FUT_DO_PROBE7(_STARPU_FUT_CODELET_DETAILS, ((job)->task)->sched_ctx, __job_size, __job_hash, (job)->task->flops / 1000, (job)->task->tag_id, workerid, ((job)->job_id)); \
+		FUT_DO_PROBE7(_STARPU_FUT_CODELET_DETAILS, ((job)->task)->sched_ctx, __job_size, __job_hash, (job)->task->flops / 1000 / ((job)->task->cl && job->task->cl->type != STARPU_SEQ ? j->task_size : 1), (job)->task->tag_id, workerid, ((job)->job_id)); \
 	}								\
 } while(0);
 
@@ -606,25 +653,31 @@ do {									\
 	FUT_DO_PROBE2(_STARPU_FUT_END_CALLBACK, job, _starpu_gettid());
 
 #define _STARPU_TRACE_JOB_PUSH(task, prio)	\
-	FUT_DO_PROBE3(_STARPU_FUT_JOB_PUSH, task, prio, _starpu_gettid());
+	FUT_DO_PROBE3(_STARPU_FUT_JOB_PUSH, _starpu_get_job_associated_to_task(task)->job_id, prio, _starpu_gettid());
 
 #define _STARPU_TRACE_JOB_POP(task, prio)	\
-	FUT_DO_PROBE3(_STARPU_FUT_JOB_POP, task, prio, _starpu_gettid());
+	FUT_DO_PROBE3(_STARPU_FUT_JOB_POP, _starpu_get_job_associated_to_task(task)->job_id, prio, _starpu_gettid());
 
 #define _STARPU_TRACE_UPDATE_TASK_CNT(counter)	\
 	FUT_DO_PROBE2(_STARPU_FUT_UPDATE_TASK_CNT, counter, _starpu_gettid())
 
 #define _STARPU_TRACE_START_FETCH_INPUT(job)	\
-	FUT_DO_PROBE2(_STARPU_FUT_START_FETCH_INPUT, job, _starpu_gettid());
+	FUT_DO_PROBE2(_STARPU_FUT_START_FETCH_INPUT_ON_TID, job, _starpu_gettid());
 
 #define _STARPU_TRACE_END_FETCH_INPUT(job)	\
-	FUT_DO_PROBE2(_STARPU_FUT_END_FETCH_INPUT, job, _starpu_gettid());
+	FUT_DO_PROBE2(_STARPU_FUT_END_FETCH_INPUT_ON_TID, job, _starpu_gettid());
 
 #define _STARPU_TRACE_START_PUSH_OUTPUT(job)	\
-	FUT_DO_PROBE2(_STARPU_FUT_START_PUSH_OUTPUT, job, _starpu_gettid());
+	FUT_DO_PROBE2(_STARPU_FUT_START_PUSH_OUTPUT_ON_TID, job, _starpu_gettid());
 
 #define _STARPU_TRACE_END_PUSH_OUTPUT(job)	\
-	FUT_DO_PROBE2(_STARPU_FUT_END_PUSH_OUTPUT, job, _starpu_gettid());
+	FUT_DO_PROBE2(_STARPU_FUT_END_PUSH_OUTPUT_ON_TID, job, _starpu_gettid());
+
+#define _STARPU_TRACE_WORKER_END_FETCH_INPUT(job, id)	\
+	FUT_DO_PROBE2(_STARPU_FUT_END_FETCH_INPUT, job, id);
+
+#define _STARPU_TRACE_WORKER_START_FETCH_INPUT(job, id)	\
+	FUT_DO_PROBE2(_STARPU_FUT_START_FETCH_INPUT, job, id);
 
 #define _STARPU_TRACE_TAG(tag, job)	\
 	FUT_DO_PROBE2(_STARPU_FUT_TAG, tag, (job)->job_id)
@@ -633,10 +686,10 @@ do {									\
 	FUT_DO_PROBE2(_STARPU_FUT_TAG_DEPS, tag_child, tag_father)
 
 #define _STARPU_TRACE_TASK_DEPS(job_prev, job_succ)	\
-	FUT_DO_PROBE2(_STARPU_FUT_TASK_DEPS, (job_prev)->job_id, (job_succ)->job_id)
+	_STARPU_FUT_DO_PROBE4STR(_STARPU_FUT_TASK_DEPS, (job_prev)->job_id, (job_succ)->job_id, (job_succ)->task->type, 1, "task")
 
-#define _STARPU_TRACE_GHOST_TASK_DEPS(ghost_prev_id, job_succ_id)		\
-	FUT_DO_PROBE2(_STARPU_FUT_TASK_DEPS, (ghost_prev_id), (job_succ_id))
+#define _STARPU_TRACE_GHOST_TASK_DEPS(ghost_prev_id, job_succ)		\
+	_STARPU_FUT_DO_PROBE4STR(_STARPU_FUT_TASK_DEPS, (ghost_prev_id), (job_succ)->job_id, (job_succ)->task->type, 1, "ghost")
 
 #define _STARPU_TRACE_TASK_NAME(job)						\
 do {										\
@@ -651,6 +704,14 @@ do {										\
 	}									\
 } while(0);
 
+#define _STARPU_TRACE_TASK_COLOR(job)						\
+do { \
+	if ((job)->task->color != 0) \
+		FUT_DO_PROBE3(_STARPU_FUT_TASK_COLOR, (job)->job_id, (job)->task->color, _starpu_gettid()); \
+	else if ((job)->task->cl && (job)->task->cl->color != 0) \
+		FUT_DO_PROBE3(_STARPU_FUT_TASK_COLOR, (job)->job_id, (job)->task->cl->color, _starpu_gettid()); \
+} while(0)
+
 #define _STARPU_TRACE_TASK_DONE(job)						\
 	FUT_DO_PROBE2(_STARPU_FUT_TASK_DONE, (job)->job_id, _starpu_gettid())
 
@@ -684,6 +745,12 @@ do {										\
 #define _STARPU_TRACE_DATA_COPY(src_node, dst_node, size)	\
 	FUT_DO_PROBE3(_STARPU_FUT_DATA_COPY, src_node, dst_node, size)
 
+#define _STARPU_TRACE_DATA_WONT_USE(handle)						\
+	FUT_DO_PROBE4(_STARPU_FUT_DATA_WONT_USE, handle, _starpu_fxt_get_submit_order(), _starpu_fxt_get_job_id(), _starpu_gettid())
+
+#define _STARPU_TRACE_DATA_DOING_WONT_USE(handle)						\
+	FUT_DO_PROBE1(_STARPU_FUT_DATA_DOING_WONT_USE, handle)
+
 #define _STARPU_TRACE_START_DRIVER_COPY(src_node, dst_node, size, com_id, prefetch, handle) \
 	FUT_DO_PROBE6(_STARPU_FUT_START_DRIVER_COPY, src_node, dst_node, size, com_id, prefetch, handle)
 
@@ -724,7 +791,7 @@ do {										\
 	FUT_DO_PROBE1(_STARPU_FUT_WORKER_SLEEP_END, _starpu_gettid());
 
 #define _STARPU_TRACE_TASK_SUBMIT(job, iter, subiter)	\
-	FUT_DO_PROBE4(_STARPU_FUT_TASK_SUBMIT, (job)->job_id, iter, subiter, _starpu_gettid());
+	FUT_DO_PROBE7(_STARPU_FUT_TASK_SUBMIT, (job)->job_id, iter, subiter, (job)->task->no_submitorder?0:_starpu_fxt_get_submit_order(), (job)->task->priority, (job)->task->type, _starpu_gettid());
 
 #define _STARPU_TRACE_TASK_SUBMIT_START()	\
 	FUT_DO_PROBE1(_STARPU_FUT_TASK_SUBMIT_START, _starpu_gettid());
@@ -780,45 +847,45 @@ do {										\
 #define _STARPU_TRACE_USER_DEFINED_END		\
 	FUT_DO_PROBE1(_STARPU_FUT_USER_DEFINED_END, _starpu_gettid());
 
-#define _STARPU_TRACE_START_ALLOC(memnode, size)		\
-	FUT_DO_PROBE3(_STARPU_FUT_START_ALLOC, memnode, _starpu_gettid(), size);
-	
-#define _STARPU_TRACE_END_ALLOC(memnode)		\
-	FUT_DO_PROBE2(_STARPU_FUT_END_ALLOC, memnode, _starpu_gettid());
+#define _STARPU_TRACE_START_ALLOC(memnode, size, handle, is_prefetch)               \
+       FUT_DO_PROBE5(_STARPU_FUT_START_ALLOC, memnode, _starpu_gettid(), size, handle, is_prefetch);
 
-#define _STARPU_TRACE_START_ALLOC_REUSE(memnode, size)		\
-	FUT_DO_PROBE3(_STARPU_FUT_START_ALLOC_REUSE, memnode, _starpu_gettid(), size);
-	
-#define _STARPU_TRACE_END_ALLOC_REUSE(memnode)		\
-	FUT_DO_PROBE2(_STARPU_FUT_END_ALLOC_REUSE, memnode, _starpu_gettid());
-	
-#define _STARPU_TRACE_START_FREE(memnode, size)		\
-	FUT_DO_PROBE3(_STARPU_FUT_START_FREE, memnode, _starpu_gettid(), size);
-	
-#define _STARPU_TRACE_END_FREE(memnode)		\
-	FUT_DO_PROBE2(_STARPU_FUT_END_FREE, memnode, _starpu_gettid());
+#define _STARPU_TRACE_END_ALLOC(memnode, handle, r)            \
+       FUT_DO_PROBE4(_STARPU_FUT_END_ALLOC, memnode, _starpu_gettid(), handle, r);
 
-#define _STARPU_TRACE_START_WRITEBACK(memnode)		\
-	FUT_DO_PROBE2(_STARPU_FUT_START_WRITEBACK, memnode, _starpu_gettid());
-	
-#define _STARPU_TRACE_END_WRITEBACK(memnode)		\
-	FUT_DO_PROBE2(_STARPU_FUT_END_WRITEBACK, memnode, _starpu_gettid());
+#define _STARPU_TRACE_START_ALLOC_REUSE(memnode, size, handle, is_prefetch)         \
+       FUT_DO_PROBE5(_STARPU_FUT_START_ALLOC_REUSE, memnode, _starpu_gettid(), size, handle, is_prefetch);
+
+#define _STARPU_TRACE_END_ALLOC_REUSE(memnode, handle, r)              \
+       FUT_DO_PROBE4(_STARPU_FUT_END_ALLOC_REUSE, memnode, _starpu_gettid(), handle, r);
+
+#define _STARPU_TRACE_START_FREE(memnode, size, handle)                \
+       FUT_DO_PROBE4(_STARPU_FUT_START_FREE, memnode, _starpu_gettid(), size, handle);
+
+#define _STARPU_TRACE_END_FREE(memnode, handle)                \
+       FUT_DO_PROBE3(_STARPU_FUT_END_FREE, memnode, _starpu_gettid(), handle);
+
+#define _STARPU_TRACE_START_WRITEBACK(memnode, handle)         \
+       FUT_DO_PROBE3(_STARPU_FUT_START_WRITEBACK, memnode, _starpu_gettid(), handle);
+
+#define _STARPU_TRACE_END_WRITEBACK(memnode, handle)           \
+       FUT_DO_PROBE3(_STARPU_FUT_END_WRITEBACK, memnode, _starpu_gettid(), handle);
 
 #define _STARPU_TRACE_USED_MEM(memnode,used)		\
 	FUT_DO_PROBE3(_STARPU_FUT_USED_MEM, memnode, used, _starpu_gettid());
-	
+
 #define _STARPU_TRACE_START_MEMRECLAIM(memnode,is_prefetch)		\
 	FUT_DO_PROBE3(_STARPU_FUT_START_MEMRECLAIM, memnode, is_prefetch, _starpu_gettid());
-	
+
 #define _STARPU_TRACE_END_MEMRECLAIM(memnode, is_prefetch)		\
 	FUT_DO_PROBE3(_STARPU_FUT_END_MEMRECLAIM, memnode, is_prefetch, _starpu_gettid());
-	
+
 #define _STARPU_TRACE_START_WRITEBACK_ASYNC(memnode)		\
 	FUT_DO_PROBE2(_STARPU_FUT_START_WRITEBACK_ASYNC, memnode, _starpu_gettid());
-	
+
 #define _STARPU_TRACE_END_WRITEBACK_ASYNC(memnode)		\
 	FUT_DO_PROBE2(_STARPU_FUT_END_WRITEBACK_ASYNC, memnode, _starpu_gettid());
-	
+
 /* We skip these events becasue they are called so often that they cause FxT to
  * fail and make the overall trace unreadable anyway. */
 #define _STARPU_TRACE_START_PROGRESS(memnode)		\
@@ -826,7 +893,7 @@ do {										\
 
 #define _STARPU_TRACE_END_PROGRESS(memnode)		\
 	FUT_DO_PROBE2(_STARPU_FUT_END_PROGRESS_ON_TID, memnode, _starpu_gettid());
-	
+
 #define _STARPU_TRACE_USER_EVENT(code)			\
 	FUT_DO_PROBE2(_STARPU_FUT_USER_EVENT, code, _starpu_gettid());
 
@@ -848,7 +915,7 @@ do {										\
 #define _STARPU_TRACE_HYPERVISOR_END() \
 	FUT_DO_PROBE1(_STARPU_FUT_HYPERVISOR_END, _starpu_gettid());
 
-#ifdef STARPU_FXT_LOCK_TRACES 
+#ifdef STARPU_FXT_LOCK_TRACES
 
 #define _STARPU_TRACE_LOCKING_MUTEX()	do { \
 	const char *file; \
@@ -986,24 +1053,24 @@ do {										\
 
 #define _STARPU_TRACE_LOCKING_MUTEX()			do {} while(0)
 #define _STARPU_TRACE_MUTEX_LOCKED()			do {} while(0)
-#define _STARPU_TRACE_UNLOCKING_MUTEX()		do {} while(0)
-#define _STARPU_TRACE_MUTEX_UNLOCKED()		do {} while(0)
+#define _STARPU_TRACE_UNLOCKING_MUTEX()			do {} while(0)
+#define _STARPU_TRACE_MUTEX_UNLOCKED()			do {} while(0)
 #define _STARPU_TRACE_TRYLOCK_MUTEX()			do {} while(0)
 #define _STARPU_TRACE_RDLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_RDLOCKED()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_RDLOCKED()			do {} while(0)
 #define _STARPU_TRACE_WRLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_WRLOCKED()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_WRLOCKED()			do {} while(0)
 #define _STARPU_TRACE_UNLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_UNLOCKED()		do {} while(0)
-#define _STARPU_TRACE_LOCKING_SPINLOCK(file, line)		do {} while(0)
-#define _STARPU_TRACE_SPINLOCK_LOCKED(file, line)		do {} while(0)
-#define _STARPU_TRACE_UNLOCKING_SPINLOCK(file, line)	do {} while(0)
-#define _STARPU_TRACE_SPINLOCK_UNLOCKED(file, line)		do {} while(0)
-#define _STARPU_TRACE_TRYLOCK_SPINLOCK(file, line)		do {} while(0)
-#define _STARPU_TRACE_COND_WAIT_BEGIN()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_UNLOCKED()			do {} while(0)
+#define _STARPU_TRACE_LOCKING_SPINLOCK(file, line)	do {(void) file; (void)line;} while(0)
+#define _STARPU_TRACE_SPINLOCK_LOCKED(file, line)	do {(void) file; (void)line;} while(0)
+#define _STARPU_TRACE_UNLOCKING_SPINLOCK(file, line)	do {(void) file; (void)line;} while(0)
+#define _STARPU_TRACE_SPINLOCK_UNLOCKED(file, line)	do {(void) file; (void)line;} while(0)
+#define _STARPU_TRACE_TRYLOCK_SPINLOCK(file, line)	do {(void) file; (void)line;} while(0)
+#define _STARPU_TRACE_COND_WAIT_BEGIN()			do {} while(0)
 #define _STARPU_TRACE_COND_WAIT_END()			do {} while(0)
 #define _STARPU_TRACE_BARRIER_WAIT_BEGIN()		do {} while(0)
-#define _STARPU_TRACE_BARRIER_WAIT_END()			do {} while(0)
+#define _STARPU_TRACE_BARRIER_WAIT_END()		do {} while(0)
 
 #endif // STARPU_FXT_LOCK_TRACES
 
@@ -1014,10 +1081,10 @@ do {										\
 	FUT_DO_PROBE2(_STARPU_FUT_DATA_LOAD, workerid, size);
 
 #define _STARPU_TRACE_START_UNPARTITION(handle, memnode)		\
-	FUT_DO_PROBE3(_STARPU_FUT_START_UNPARTITION, memnode, _starpu_gettid(), handle);
-	
+	FUT_DO_PROBE3(_STARPU_FUT_START_UNPARTITION_ON_TID, memnode, _starpu_gettid(), handle);
+
 #define _STARPU_TRACE_END_UNPARTITION(handle, memnode)		\
-	FUT_DO_PROBE3(_STARPU_FUT_END_UNPARTITION, memnode, _starpu_gettid(), handle);
+	FUT_DO_PROBE3(_STARPU_FUT_END_UNPARTITION_ON_TID, memnode, _starpu_gettid(), handle);
 
 #define _STARPU_TRACE_SCHED_COMPONENT_PUSH_PRIO(workerid, ntasks, exp_len)		\
 	FUT_DO_PROBE4(_STARPU_FUT_SCHED_COMPONENT_PUSH_PRIO, _starpu_gettid(), workerid, ntasks, exp_len);
@@ -1051,56 +1118,66 @@ do {										\
 #define _STARPU_TRACE_HANDLE_DATA_UNREGISTER(handle)	\
 	FUT_DO_PROBE1(_STARPU_FUT_HANDLE_DATA_UNREGISTER, handle)
 
-#if 0
-#define _STARPU_TRACE_DATA_INVALIDATE(handle, node)		\
-	FUT_DO_PROBE2(_STARPU_FUT_DATA_INVALIDATE, handle, node)
-#else
-#define _STARPU_TRACE_DATA_INVALIDATE(handle, node)	do {} while (0)
-#endif
+//Coherency Data Traces
+#define _STARPU_TRACE_DATA_STATE_INVALID(handle, node)      \
+       FUT_DO_PROBE2(_STARPU_FUT_DATA_STATE_INVALID, handle, node)
+
+#define _STARPU_TRACE_DATA_STATE_OWNER(handle, node)           \
+       FUT_DO_PROBE2(_STARPU_FUT_DATA_STATE_OWNER, handle, node)
+
+#define _STARPU_TRACE_DATA_STATE_SHARED(handle, node)          \
+       FUT_DO_PROBE2(_STARPU_FUT_DATA_STATE_SHARED, handle, node)
+
+#define _STARPU_TRACE_DATA_REQUEST_CREATED(handle, orig, dest, prio, is_pre)          \
+       FUT_DO_PROBE5(_STARPU_FUT_DATA_REQUEST_CREATED, orig, dest, prio, handle, is_pre)
+
 
 #else // !STARPU_USE_FXT
 
 /* Dummy macros in case FxT is disabled */
-#define _STARPU_TRACE_NEW_MEM_NODE(nodeid)	do {} while(0)
-#define _STARPU_TRACE_WORKER_INIT_START(a,b,c,d,e,f)	do {} while(0)
-#define _STARPU_TRACE_WORKER_INIT_END(workerid)	do {} while(0)
-#define _STARPU_TRACE_START_CODELET_BODY(job, nimpl, perf_arch, workerid)	do {} while(0)
-#define _STARPU_TRACE_END_CODELET_BODY(job, nimpl, perf_arch, workerid)	do {} while(0)
-#define _STARPU_TRACE_START_EXECUTING()	do {} while(0)
-#define _STARPU_TRACE_END_EXECUTING()	do {} while(0)
-#define _STARPU_TRACE_START_CALLBACK(job)	do {} while(0)
-#define _STARPU_TRACE_END_CALLBACK(job)		do {} while(0)
-#define _STARPU_TRACE_JOB_PUSH(task, prio)	do {} while(0)
-#define _STARPU_TRACE_JOB_POP(task, prio)	do {} while(0)
-#define _STARPU_TRACE_UPDATE_TASK_CNT(counter)	do {} while(0)
-#define _STARPU_TRACE_START_FETCH_INPUT(job)	do {} while(0)
-#define _STARPU_TRACE_END_FETCH_INPUT(job)	do {} while(0)
-#define _STARPU_TRACE_START_PUSH_OUTPUT(job)	do {} while(0)
-#define _STARPU_TRACE_END_PUSH_OUTPUT(job)	do {} while(0)
-#define _STARPU_TRACE_TAG(tag, job)	do {} while(0)
-#define _STARPU_TRACE_TAG_DEPS(a, b)	do {} while(0)
-#define _STARPU_TRACE_TASK_DEPS(a, b)		do {} while(0)
-#define _STARPU_TRACE_GHOST_TASK_DEPS(a, b)	do {} while(0)
-#define _STARPU_TRACE_TASK_NAME(a)		do {} while(0)
-#define _STARPU_TRACE_TASK_DONE(a)		do {} while(0)
-#define _STARPU_TRACE_TAG_DONE(a)		do {} while(0)
-#define _STARPU_TRACE_DATA_NAME(a, b)		do {} while(0)
-#define _STARPU_TRACE_DATA_COORDINATES(a, b, c)		do {} while(0)
-#define _STARPU_TRACE_DATA_COPY(a, b, c)		do {} while(0)
-#define _STARPU_TRACE_START_DRIVER_COPY(a,b,c,d,e,f)	do {} while(0)
-#define _STARPU_TRACE_END_DRIVER_COPY(a,b,c,d,e)	do {} while(0)
-#define _STARPU_TRACE_START_DRIVER_COPY_ASYNC(a,b)	do {} while(0)
-#define _STARPU_TRACE_END_DRIVER_COPY_ASYNC(a,b)	do {} while(0)
-#define _STARPU_TRACE_WORK_STEALING(a, b)	do {} while(0)
-#define _STARPU_TRACE_WORKER_DEINIT_START	do {} while(0)
-#define _STARPU_TRACE_WORKER_DEINIT_END(a)	do {} while(0)
+#define _STARPU_TRACE_NEW_MEM_NODE(nodeid)		do {(void)(nodeid);} while(0)
+#define _STARPU_TRACE_WORKER_INIT_START(a,b,c,d,e,f)	do {(void)(a); (void)(b); (void)(c); (void)(d); (void)(e); (void)(f);} while(0)
+#define _STARPU_TRACE_WORKER_INIT_END(workerid)		do {(void)(workerid);} while(0)
+#define _STARPU_TRACE_START_CODELET_BODY(job, nimpl, perf_arch, workerid) 	do {(void)(job); (void)(nimpl); (void)(perf_arch); (void)(workerid);} while(0)
+#define _STARPU_TRACE_END_CODELET_BODY(job, nimpl, perf_arch, workerid)		do {(void)(job); (void)(nimpl); (void)(perf_arch); (void)(workerid);} while(0)
+#define _STARPU_TRACE_START_EXECUTING()		do {} while(0)
+#define _STARPU_TRACE_END_EXECUTING()		do {} while(0)
+#define _STARPU_TRACE_START_CALLBACK(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_END_CALLBACK(job)		do {(void)(job);} while(0)
+#define _STARPU_TRACE_JOB_PUSH(task, prio)	do {(void)(task); (void)(prio);} while(0)
+#define _STARPU_TRACE_JOB_POP(task, prio)	do {(void)(task); (void)(prio);} while(0)
+#define _STARPU_TRACE_UPDATE_TASK_CNT(counter)	do {(void)(counter);} while(0)
+#define _STARPU_TRACE_START_FETCH_INPUT(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_END_FETCH_INPUT(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_START_PUSH_OUTPUT(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_END_PUSH_OUTPUT(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_TAG(tag, job)		do {(void)(tag); (void)(job);} while(0)
+#define _STARPU_TRACE_TAG_DEPS(a, b)		do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_TASK_DEPS(a, b)		do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_GHOST_TASK_DEPS(a, b)	do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_TASK_NAME(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_TASK_COLOR(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_TASK_DONE(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_TAG_DONE(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_DATA_NAME(a, b)		do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_DATA_COORDINATES(a, b, c)	do {(void)(a); (void)(b); (void)(c);} while(0)
+#define _STARPU_TRACE_DATA_COPY(a, b, c)		do {(void)(a); (void)(b); (void)(c);} while(0)
+#define _STARPU_TRACE_DATA_WONT_USE(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_DATA_DOING_WONT_USE(a)		do {(void)(a);} while(0)
+#define _STARPU_TRACE_START_DRIVER_COPY(a,b,c,d,e,f)	do {(void)(a); (void)(b); (void)(c); (void)(d); (void)(e); (void)(f);} while(0)
+#define _STARPU_TRACE_END_DRIVER_COPY(a,b,c,d,e)	do {(void)(a); (void)(b); (void)(c); (void)(d); (void)(e);} while(0)
+#define _STARPU_TRACE_START_DRIVER_COPY_ASYNC(a,b)	do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_END_DRIVER_COPY_ASYNC(a,b)	do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_WORK_STEALING(a, b)		do {(void)(a); (void)(b);} while(0)
+#define _STARPU_TRACE_WORKER_DEINIT_START		do {} while(0)
+#define _STARPU_TRACE_WORKER_DEINIT_END(a)		do {(void)(a);} while(0)
 #define _STARPU_TRACE_WORKER_SCHEDULING_START		do {} while(0)
 #define _STARPU_TRACE_WORKER_SCHEDULING_END		do {} while(0)
 #define _STARPU_TRACE_WORKER_SCHEDULING_PUSH		do {} while(0)
 #define _STARPU_TRACE_WORKER_SCHEDULING_POP		do {} while(0)
 #define _STARPU_TRACE_WORKER_SLEEP_START		do {} while(0)
-#define _STARPU_TRACE_WORKER_SLEEP_END		do {} while(0)
-#define _STARPU_TRACE_TASK_SUBMIT(job, a, b)		do {} while(0)
+#define _STARPU_TRACE_WORKER_SLEEP_END			do {} while(0)
+#define _STARPU_TRACE_TASK_SUBMIT(job, a, b)			do {(void)(job); (void)(a);(void)(b);} while(0)
 #define _STARPU_TRACE_TASK_SUBMIT_START()		do {} while(0)
 #define _STARPU_TRACE_TASK_SUBMIT_END()			do {} while(0)
 #define _STARPU_TRACE_TASK_THROTTLE_START()		do {} while(0)
@@ -1113,67 +1190,72 @@ do {										\
 #define _STARPU_TRACE_TASK_MPI_PRE_END()		do {} while(0)
 #define _STARPU_TRACE_TASK_MPI_POST_START()		do {} while(0)
 #define _STARPU_TRACE_TASK_MPI_POST_END()		do {} while(0)
-#define _STARPU_TRACE_TASK_WAIT_START(job)		do {} while(0)
+#define _STARPU_TRACE_TASK_WAIT_START(job)		do {(void)(job);} while(0)
 #define _STARPU_TRACE_TASK_WAIT_END()			do {} while(0)
 #define _STARPU_TRACE_TASK_WAIT_FOR_ALL_START()		do {} while(0)
 #define _STARPU_TRACE_TASK_WAIT_FOR_ALL_END()		do {} while(0)
-#define _STARPU_TRACE_USER_DEFINED_START		do {} while(0)
-#define _STARPU_TRACE_USER_DEFINED_END		do {} while(0)
-#define _STARPU_TRACE_START_ALLOC(memnode, size)	do {} while(0)
-#define _STARPU_TRACE_END_ALLOC(memnode)		do {} while(0)
-#define _STARPU_TRACE_START_ALLOC_REUSE(a, size)	do {} while(0)
-#define _STARPU_TRACE_END_ALLOC_REUSE(a)		do {} while(0)
-#define _STARPU_TRACE_START_FREE(memnode, size)	do {} while(0)
-#define _STARPU_TRACE_END_FREE(memnode)		do {} while(0)
-#define _STARPU_TRACE_START_WRITEBACK(memnode)	do {} while(0)
-#define _STARPU_TRACE_END_WRITEBACK(memnode)		do {} while(0)
-#define _STARPU_TRACE_USED_MEM(memnode,used)		do {} while (0)
-#define _STARPU_TRACE_START_MEMRECLAIM(memnode,is_prefetch)	do {} while(0)
-#define _STARPU_TRACE_END_MEMRECLAIM(memnode,is_prefetch)	do {} while(0)
-#define _STARPU_TRACE_START_WRITEBACK_ASYNC(memnode)	do {} while(0)
-#define _STARPU_TRACE_END_WRITEBACK_ASYNC(memnode)	do {} while(0)
-#define _STARPU_TRACE_START_PROGRESS(memnode)	do {} while(0)
-#define _STARPU_TRACE_END_PROGRESS(memnode)	do {} while(0)
-#define _STARPU_TRACE_USER_EVENT(code)		do {} while(0)
-#define _STARPU_TRACE_SET_PROFILING(status)	do {} while(0)
-#define _STARPU_TRACE_TASK_WAIT_FOR_ALL		do {} while(0)
-#define _STARPU_TRACE_EVENT(S)		do {} while(0)
-#define _STARPU_TRACE_THREAD_EVENT(S)		do {} while(0)
+#define _STARPU_TRACE_USER_DEFINED_START()		do {} while(0)
+#define _STARPU_TRACE_USER_DEFINED_END()		do {} while(0)
+#define _STARPU_TRACE_START_ALLOC(memnode, size, handle, is_prefetch)       do {(void)(memnode); (void)(size); (void)(handle);} while(0)
+#define _STARPU_TRACE_END_ALLOC(memnode, handle, r)            do {(void)(memnode); (void)(handle); (void)(r);} while(0)
+#define _STARPU_TRACE_START_ALLOC_REUSE(a, size, handle, is_prefetch)       do {(void)(a); (void)(size); (void)(handle);} while(0)
+#define _STARPU_TRACE_END_ALLOC_REUSE(a, handle, r)            do {(void)(a); (void)(handle); (void)(r);} while(0)
+#define _STARPU_TRACE_START_FREE(memnode, size, handle)                do {(void)(memnode); (void)(size); (void)(handle);} while(0)
+#define _STARPU_TRACE_END_FREE(memnode, handle)                        do {(void)(memnode); (void)(handle);} while(0)
+#define _STARPU_TRACE_START_WRITEBACK(memnode, handle)         do {(void)(memnode); (void)(handle);} while(0)
+#define _STARPU_TRACE_END_WRITEBACK(memnode, handle)           do {(void)(memnode); (void)(handle);} while(0)
+#define _STARPU_TRACE_USED_MEM(memnode,used)		do {(void)(memnode); (void)(used);} while (0)
+#define _STARPU_TRACE_START_MEMRECLAIM(memnode,is_prefetch)	do {(void)(memnode); (void)(is_prefetch);} while(0)
+#define _STARPU_TRACE_END_MEMRECLAIM(memnode,is_prefetch)	do {(void)(memnode); (void)(is_prefetch);} while(0)
+#define _STARPU_TRACE_START_WRITEBACK_ASYNC(memnode)	do {(void)(memnode);} while(0)
+#define _STARPU_TRACE_END_WRITEBACK_ASYNC(memnode)	do {(void)(memnode);} while(0)
+#define _STARPU_TRACE_START_PROGRESS(memnode)		do {(void)( memnode);} while(0)
+#define _STARPU_TRACE_END_PROGRESS(memnode)		do {(void)( memnode);} while(0)
+#define _STARPU_TRACE_USER_EVENT(code)			do {(void)(code);} while(0)
+#define _STARPU_TRACE_SET_PROFILING(status)		do {(void)(status);} while(0)
+#define _STARPU_TRACE_TASK_WAIT_FOR_ALL()		do {} while(0)
+#define _STARPU_TRACE_EVENT(S)				do {(void)(S);} while(0)
+#define _STARPU_TRACE_THREAD_EVENT(S)			do {(void)(S);} while(0)
 #define _STARPU_TRACE_LOCKING_MUTEX()			do {} while(0)
 #define _STARPU_TRACE_MUTEX_LOCKED()			do {} while(0)
-#define _STARPU_TRACE_UNLOCKING_MUTEX()		do {} while(0)
-#define _STARPU_TRACE_MUTEX_UNLOCKED()		do {} while(0)
+#define _STARPU_TRACE_UNLOCKING_MUTEX()			do {} while(0)
+#define _STARPU_TRACE_MUTEX_UNLOCKED()			do {} while(0)
 #define _STARPU_TRACE_TRYLOCK_MUTEX()			do {} while(0)
 #define _STARPU_TRACE_RDLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_RDLOCKED()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_RDLOCKED()			do {} while(0)
 #define _STARPU_TRACE_WRLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_WRLOCKED()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_WRLOCKED()			do {} while(0)
 #define _STARPU_TRACE_UNLOCKING_RWLOCK()		do {} while(0)
-#define _STARPU_TRACE_RWLOCK_UNLOCKED()		do {} while(0)
-#define _STARPU_TRACE_LOCKING_SPINLOCK(file, line)		do {} while(0)
-#define _STARPU_TRACE_SPINLOCK_LOCKED(file, line)		do {} while(0)
-#define _STARPU_TRACE_UNLOCKING_SPINLOCK(file, line)	do {} while(0)
-#define _STARPU_TRACE_SPINLOCK_UNLOCKED(file, line)		do {} while(0)
-#define _STARPU_TRACE_TRYLOCK_SPINLOCK(file, line)		do {} while(0)
-#define _STARPU_TRACE_COND_WAIT_BEGIN()		do {} while(0)
+#define _STARPU_TRACE_RWLOCK_UNLOCKED()			do {} while(0)
+#define _STARPU_TRACE_LOCKING_SPINLOCK(file, line)	do {(void)(file); (void)(line);} while(0)
+#define _STARPU_TRACE_SPINLOCK_LOCKED(file, line)	do {(void)(file); (void)(line);} while(0)
+#define _STARPU_TRACE_UNLOCKING_SPINLOCK(file, line)	do {(void)(file); (void)(line);} while(0)
+#define _STARPU_TRACE_SPINLOCK_UNLOCKED(file, line)	do {(void)(file); (void)(line);} while(0)
+#define _STARPU_TRACE_TRYLOCK_SPINLOCK(file, line)	do {(void)(file); (void)(line);} while(0)
+#define _STARPU_TRACE_COND_WAIT_BEGIN()			do {} while(0)
 #define _STARPU_TRACE_COND_WAIT_END()			do {} while(0)
 #define _STARPU_TRACE_BARRIER_WAIT_BEGIN()		do {} while(0)
-#define _STARPU_TRACE_BARRIER_WAIT_END()			do {} while(0)
-#define _STARPU_TRACE_MEMORY_FULL(size)				do {} while(0)
-#define _STARPU_TRACE_DATA_LOAD(workerid,size)		do {} while(0)
-#define _STARPU_TRACE_START_UNPARTITION(handle, memnode)	do {} while(0)
-#define _STARPU_TRACE_END_UNPARTITION(handle, memnode)		do {} while(0)
-#define _STARPU_TRACE_SCHED_COMPONENT_PUSH_PRIO(workerid, ntasks, exp_len)	do {} while(0)
-#define _STARPU_TRACE_SCHED_COMPONENT_POP_PRIO(workerid, ntasks, exp_len)	do {} while(0)
-#define _STARPU_TRACE_HYPERVISOR_BEGIN()        do {} while(0)
+#define _STARPU_TRACE_BARRIER_WAIT_END()		do {} while(0)
+#define _STARPU_TRACE_MEMORY_FULL(size)			do {(void)(size);} while(0)
+#define _STARPU_TRACE_DATA_LOAD(workerid,size)		do {(void)(workerid); (void)(size);} while(0)
+#define _STARPU_TRACE_START_UNPARTITION(handle, memnode) 	do {(void)(handle); (void)(memnode);} while(0)
+#define _STARPU_TRACE_END_UNPARTITION(handle, memnode)		do {(void)(handle); (void)(memnode);} while(0)
+#define _STARPU_TRACE_SCHED_COMPONENT_PUSH_PRIO(workerid, ntasks, exp_len)	do {(void)(workerid); (void)(ntasks); (void)(exp_len);} while(0)
+#define _STARPU_TRACE_SCHED_COMPONENT_POP_PRIO(workerid, ntasks, exp_len)	do {(void)(workerid); (void)(ntasks); (void)(exp_len);} while(0)
+#define _STARPU_TRACE_HYPERVISOR_BEGIN()        	do {} while(0)
 #define _STARPU_TRACE_HYPERVISOR_END()                  do {} while(0)
-#define _STARPU_TRACE_SCHED_COMPONENT_NEW(component)	do {} while (0)
-#define _STARPU_TRACE_SCHED_COMPONENT_CONNECT(parent, child)	do {} while (0)
-#define _STARPU_TRACE_SCHED_COMPONENT_PUSH(from, to, task)	do {} while (0)
-#define _STARPU_TRACE_SCHED_COMPONENT_PULL(from, to, task)	do {} while (0)
-#define _STARPU_TRACE_HANDLE_DATA_REGISTER(handle)	do {} while (0)
-#define _STARPU_TRACE_HANDLE_DATA_UNREGISTER(handle)	do {} while (0)
-#define _STARPU_TRACE_DATA_INVALIDATE(handle, node)	do {} while (0)
+#define _STARPU_TRACE_SCHED_COMPONENT_NEW(component)	do {(void)(component);} while (0)
+#define _STARPU_TRACE_SCHED_COMPONENT_CONNECT(parent, child)	do {(void)(parent); (void)(child);} while (0)
+#define _STARPU_TRACE_SCHED_COMPONENT_PUSH(from, to, task)	do {(void)(from); (void)(to); (void)(task);} while (0)
+#define _STARPU_TRACE_SCHED_COMPONENT_PULL(from, to, task)	do {(void)(from); (void)(to); (void)(task);} while (0)
+#define _STARPU_TRACE_HANDLE_DATA_REGISTER(handle)	do {(void)(handle);} while (0)
+#define _STARPU_TRACE_HANDLE_DATA_UNREGISTER(handle)	do {(void)(handle);} while (0)
+#define _STARPU_TRACE_WORKER_START_FETCH_INPUT(job, id)	do {(void)(job); (void)(id);} while(0)
+#define _STARPU_TRACE_WORKER_END_FETCH_INPUT(job, id)	do {(void)(job); (void)(id);} while(0)
+#define _STARPU_TRACE_DATA_STATE_INVALID(handle, node)	do {(void)(handle); (void)(node);} while(0)
+#define _STARPU_TRACE_DATA_STATE_OWNER(handle, node)	do {(void)(handle); (void)(node);} while(0)
+#define _STARPU_TRACE_DATA_STATE_SHARED(handle, node)	do {(void)(handle); (void)(node);} while(0)
+#define _STARPU_TRACE_DATA_REQUEST_CREATED(handle, orig, dest, prio, is_pre) do {(void)(handle); (void)(orig); (void)(dest); (void)(prio); (void)(is_pre);} while(0)
 
 #endif // STARPU_USE_FXT
 
