@@ -1,10 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2016                                Inria
- * Copyright (C) 2008-2017                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,13 +26,9 @@
 #include <common/graph.h>
 #include <profiling/profiling.h>
 #include <profiling/bound.h>
-#include <starpu_top.h>
-#include <top/starpu_top_core.h>
 #include <core/debug.h>
 #include <limits.h>
 
-/* we need to identify each task to generate the DAG. */
-static unsigned long job_cnt = 0;
 static int max_memory_use;
 static unsigned long njobs, maxnjobs;
 
@@ -48,7 +42,7 @@ void _starpu_job_init(void)
 {
 	max_memory_use = starpu_get_env_number_default("STARPU_MAX_MEMORY_USE", 0);
 #ifdef STARPU_DEBUG
-	_starpu_job_multilist_init_all_submitted(&all_jobs_list);
+	_starpu_job_multilist_head_init_all_submitted(&all_jobs_list);
 #endif
 }
 
@@ -89,12 +83,10 @@ struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_tas
 	job->task = task;
 
 #ifndef STARPU_USE_FXT
-	if (_starpu_bound_recording || _starpu_top_status_get() ||
-		_starpu_task_break_on_push != -1 || _starpu_task_break_on_sched != -1 || _starpu_task_break_on_pop != -1 || _starpu_task_break_on_exec != -1
-		|| STARPU_AYU_EVENT)
+	if (_starpu_bound_recording || _starpu_task_break_on_push != -1 || _starpu_task_break_on_sched != -1 || _starpu_task_break_on_pop != -1 || _starpu_task_break_on_exec != -1 || STARPU_AYU_EVENT)
 #endif
 	{
-		job->job_id = STARPU_ATOMIC_ADDL(&job_cnt, 1);
+		job->job_id = _starpu_fxt_get_job_id();
 		STARPU_AYU_ADDTASK(job->job_id, task);
 		STARPU_ASSERT(job->job_id != ULONG_MAX);
 	}
@@ -187,7 +179,7 @@ void _starpu_wait_job(struct _starpu_job *j)
 	{
 		STARPU_PTHREAD_COND_WAIT(&j->sync_cond, &j->sync_mutex);
 	}
-	
+
 	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
         _STARPU_LOG_OUT();
 }
@@ -263,9 +255,33 @@ void _starpu_handle_job_submission(struct _starpu_job *j)
 #endif
 }
 
+void starpu_task_end_dep_release(struct starpu_task *t)
+{
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(t);
+	_starpu_handle_job_termination(j);
+}
+
+void starpu_task_end_dep_add(struct starpu_task *t, int nb_deps)
+{
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(t);
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	t->nb_termination_call_required += nb_deps;
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+}
+
 void _starpu_handle_job_termination(struct _starpu_job *j)
 {
+	if (j->task->nb_termination_call_required != 0)
+	{
+		STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+		int nb = j->task->nb_termination_call_required;
+		j->task->nb_termination_call_required -= 1;
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		if (nb != 0) return;
+	}
+
 	struct starpu_task *task = j->task;
+	struct starpu_task *end_rdep = NULL;
 	unsigned sched_ctx = task->sched_ctx;
 	double flops = task->flops;
 	const unsigned continuation =
@@ -275,12 +291,6 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		0
 #endif
 		;
-	/* Read cl fields before releasing dependencies, for the case of a
-	 * switch_cl which is freed by data_unregister happening as soon as
-	 * the dependencies are released.
-	 */
-	unsigned nowhere = !task->cl || task->cl->where == STARPU_NOWHERE;
-
 #ifdef STARPU_DEBUG
 	STARPU_PTHREAD_MUTEX_LOCK(&all_jobs_list_mutex);
 	_starpu_job_multilist_erase_all_submitted(&all_jobs_list, j);
@@ -288,6 +298,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 #endif
 
 	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	STARPU_ASSERT(task->status == STARPU_TASK_RUNNING);
 #ifdef STARPU_OPENMP
 	if (continuation)
 	{
@@ -303,6 +314,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		 * function. A value of 1 means that the codelet was executed but that
 		 * the callback is not done yet. */
 		j->terminated = 1;
+		end_rdep = j->end_rdep;
 	}
 	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
@@ -334,6 +346,9 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 				_starpu_spin_unlock(&handle->header_lock);
 		}
 	}
+	/* Check nowhere before releasing the sequential consistency (which may
+	 * unregister the handle and free its switch_cl, and thus task->cl here.  */
+	unsigned nowhere = !task->cl || task->cl->where == STARPU_NOWHERE || task->where == STARPU_NOWHERE;
 	/* If this is a continuation, we do not release task dependencies now.
 	 * Task dependencies will be released only when the continued task
 	 * fully completes */
@@ -344,6 +359,30 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		_starpu_release_task_enforce_sequential_consistency(j);
 	}
 
+	/* If the job was executed on a combined worker there is no need for the
+	 * scheduler to process it : the task structure doesn't contain any valuable
+	 * data as it's not linked to an actual worker */
+	/* control task should not execute post_exec_hook */
+	if(j->task_size == 1 && !nowhere && !j->internal
+#ifdef STARPU_OPENMP
+	/* If this is a continuation, we do not execute the post_exec_hook. The
+	 * post_exec_hook will be run only when the continued task fully
+	 * completes.
+	 *
+	 * Note: If needed, a specific hook could be added to handle stopped
+	 * tasks */
+	&& !continuation
+#endif
+			)
+	{
+		_starpu_sched_post_exec_hook(task);
+#ifdef STARPU_USE_SC_HYPERVISOR
+		int workerid = starpu_worker_get_id();
+		_starpu_sched_ctx_post_exec_task_cb(workerid, task, data_size, j->footprint);
+#endif //STARPU_USE_SC_HYPERVISOR
+
+	}
+
 	/* Remove ourself from the graph before notifying dependencies */
 	if (_starpu_graph_record)
 		_starpu_graph_drop_job(j);
@@ -352,7 +391,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	 * to tell them that we will not exist any more before notifying the
 	 * tasks waiting for us
 	 *
-	 * For continuations, implicit dependency handles are only released 
+	 * For continuations, implicit dependency handles are only released
 	 * when the task fully completes */
 	if (j->implicit_dep_handle && !continuation)
 	{
@@ -373,6 +412,8 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	if (!continuation)
 	{
 		/* in case there are dependencies, wake up the proper tasks */
+		if (end_rdep)
+			starpu_task_end_dep_release(end_rdep);
 		_starpu_notify_dependencies(j);
 	}
 
@@ -414,30 +455,6 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		}
 	}
 
-	/* If the job was executed on a combined worker there is no need for the
-	 * scheduler to process it : the task structure doesn't contain any valuable
-	 * data as it's not linked to an actual worker */
-	/* control task should not execute post_exec_hook */
-	if(j->task_size == 1 && !nowhere && !j->internal
-#ifdef STARPU_OPENMP
-	/* If this is a continuation, we do not execute the post_exec_hook. The
-	 * post_exec_hook will be run only when the continued task fully
-	 * completes.
-	 *
-	 * Note: If needed, a specific hook could be added to handle stopped
-	 * tasks */
-	&& !continuation
-#endif
-			)
-	{
-		_starpu_sched_post_exec_hook(task);
-#ifdef STARPU_USE_SC_HYPERVISOR
-		int workerid = starpu_worker_get_id();
-		_starpu_sched_ctx_post_exec_task_cb(workerid, task, data_size, j->footprint);
-#endif //STARPU_USE_SC_HYPERVISOR
-
-	}
-
 	/* Note: For now, we keep the TASK_DONE trace event for continuation,
 	 * however we could add a specific event for stopped tasks if needed.
 	 */
@@ -449,6 +466,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	unsigned destroy = task->destroy;
 	unsigned detach = task->detach;
 	unsigned regenerate = task->regenerate;
+	unsigned synchronous = task->synchronous;
 
 	/* we do not desallocate the job structure if some is going to
 	 * wait after the task */
@@ -484,9 +502,9 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	/* A continuation is not much different from a regenerated task. */
 	if (regenerate || continuation)
 	{
-		STARPU_ASSERT_MSG((detach && !destroy && !task->synchronous)
+		STARPU_ASSERT_MSG((detach && !destroy && !synchronous)
 				|| continuation
-				, "Regenerated task must be detached (was %u), and not have detroy=1 (was %u) or synchronous=1 (was %u)", detach, destroy, task->synchronous);
+				, "Regenerated task must be detached (was %u), and not have detroy=1 (was %u) or synchronous=1 (was %u)", detach, destroy, synchronous);
 		STARPU_AYU_ADDTASK(j->job_id, j->exclude_from_dag?NULL:task);
 
 		{
@@ -501,6 +519,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 #endif
 			{
 				/* We reuse the same job structure */
+				task->status = STARPU_TASK_BLOCKED;
 				int ret = _starpu_submit_job(j);
 				STARPU_ASSERT(!ret);
 			}
@@ -580,6 +599,7 @@ static unsigned _starpu_not_all_task_deps_are_fulfilled(struct _starpu_job *j)
 
 	if (!j->submitted || (job_successors->ndeps != job_successors->ndeps_completed))
 	{
+		STARPU_ASSERT(j->task->status == STARPU_TASK_BLOCKED || j->task->status == STARPU_TASK_BLOCKED_ON_TAG);
                 j->task->status = STARPU_TASK_BLOCKED_ON_TASK;
 		ret = 1;
 	}
@@ -681,6 +701,24 @@ unsigned _starpu_reenforce_task_deps_and_schedule(struct _starpu_job *j)
 }
 #endif
 
+/* This is called when a tag or task dependency is to be released.  */
+void _starpu_enforce_deps_notify_job_ready_soon(struct _starpu_job *j, _starpu_notify_job_start_data *data, int tag)
+{
+	if (!j->submitted)
+		/* It's not even submitted actually */
+		return;
+	struct _starpu_cg_list *job_successors = &j->job_successors;
+        /* tag is 1 when we got woken up by a tag dependency about to be
+         * released, and thus we have to check the exact numbner of
+         * dependencies.  Otherwise it's a task dependency which is about to be
+         * released.  */
+	if (job_successors->ndeps != job_successors->ndeps_completed + 1 - tag)
+		/* There are still other dependencies */
+		return;
+
+	_starpu_enforce_data_deps_notify_job_ready_soon(j, data);
+}
+
 /* Ordered tasks are simply recorded as they arrive in the local_ordered_tasks
  * ring buffer, indexed by order, and pulled from its head. */
 /* TODO: replace with perhaps a heap */
@@ -716,10 +754,10 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 {
 	/* Check that the worker is able to execute the task ! */
 	STARPU_ASSERT(task && task->cl);
-	if (STARPU_UNLIKELY(!(worker->worker_mask & task->cl->where)))
+	if (STARPU_UNLIKELY(!(worker->worker_mask & task->where)))
 		return -ENODEV;
 
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
+	starpu_worker_lock(worker->workerid);
 
 	if (task->execute_on_a_specific_worker && task->workerorder)
 	{
@@ -755,6 +793,9 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 	}
 	else
 	{
+#ifdef STARPU_DEVEL
+#warning FIXME use a prio_list
+#endif
 		if (prio)
 			starpu_task_list_push_front(&worker->local_tasks, task);
 		else
@@ -763,7 +804,7 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 
 	starpu_wake_worker_locked(worker->workerid);
 	starpu_push_task_end(task);
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
+	starpu_worker_unlock(worker->workerid);
 
 	return 0;
 }
