@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013,2016,2017                      Inria
- * Copyright (C) 2008-2019                                Université de Bordeaux
- * Copyright (C) 2018                                     Federal University of Rio Grande do Sul (UFRGS)
- * Copyright (C) 2010-2017,2019                           CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2018       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -115,14 +113,71 @@ static unsigned tidying[STARPU_MAXNODES];
 /* Whether some thread is currently reclaiming memory for this node */
 static unsigned reclaiming[STARPU_MAXNODES];
 
+/* This records that we tried to prefetch data but went out of memory, so will
+ * probably fail again to prefetch data, thus not trace each and every
+ * attempt. */
+static volatile int prefetch_out_of_memory[STARPU_MAXNODES];
+
 int _starpu_is_reclaiming(unsigned node)
 {
+	STARPU_ASSERT(node < STARPU_MAXNODES);
 	return tidying[node] || reclaiming[node];
 }
 
+/* Whether this memory node can evict data to another node */
+static unsigned evictable[STARPU_MAXNODES];
+
+static int can_evict(unsigned node)
+{
+	return evictable[node];
+}
+
+/* Called after initializing the set of memory nodes */
+/* We use an accelerator -> CPU RAM -> disk storage hierarchy */
+void _starpu_mem_chunk_init_last(void)
+{
+	unsigned disk = 0;
+	unsigned nnodes = starpu_memory_nodes_get_count(), i;
+
+	for (i = 0; i < nnodes; i++)
+	{
+		enum starpu_node_kind kind = starpu_node_get_kind(i);
+
+		if (kind == STARPU_DISK_RAM)
+			/* Some disk, will be able to evict RAM */
+			/* TODO: disk hierarchy */
+			disk = 1;
+
+		else if (kind != STARPU_CPU_RAM)
+			/* This is an accelerator, we can evict to main RAM */
+			evictable[i] = 1;
+	}
+
+	if (disk)
+		for (i = 0; i < nnodes; i++)
+		{
+			enum starpu_node_kind kind = starpu_node_get_kind(i);
+			if (kind == STARPU_CPU_RAM)
+				evictable[i] = 1;
+		}
+}
+
+/* A disk was registered, RAM is now evictable */
+void _starpu_mem_chunk_disk_register(unsigned disk_memnode)
+{
+	(void) disk_memnode;
+	unsigned nnodes = starpu_memory_nodes_get_count(), i;
+
+	for (i = 0; i < nnodes; i++)
+	{
+		enum starpu_node_kind kind = starpu_node_get_kind(i);
+		if (kind == STARPU_CPU_RAM)
+			evictable[i] = 1;
+	}
+}
+
 static int get_better_disk_can_accept_size(starpu_data_handle_t handle, unsigned node);
 static int choose_target(starpu_data_handle_t handle, unsigned node);
-static int can_evict(unsigned node);
 
 void _starpu_init_mem_chunk_lists(void)
 {
@@ -134,6 +189,7 @@ void _starpu_init_mem_chunk_lists(void)
 		STARPU_HG_DISABLE_CHECKING(mc_cache_size[i]);
 		STARPU_HG_DISABLE_CHECKING(mc_nb[i]);
 		STARPU_HG_DISABLE_CHECKING(mc_clean_nb[i]);
+		STARPU_HG_DISABLE_CHECKING(prefetch_out_of_memory[i]);
 	}
 	/* We do not enable forcing available memory by default, since
 	  this makes StarPU spuriously free data when prefetching fills the
@@ -507,7 +563,8 @@ static size_t try_to_throw_mem_chunk(struct _starpu_mem_chunk *mc, unsigned node
 		return 0;
 
 	/* This data cannnot be pushed outside CPU memory */
-	if (!handle->ooc && starpu_node_get_kind(node) == STARPU_CPU_RAM)
+	if (!handle->ooc && starpu_node_get_kind(node) == STARPU_CPU_RAM
+		&& starpu_memory_nodes_get_numa_count() == 1)
 		return 0;
 
 	if (diduse_barrier && !mc->diduse)
@@ -990,6 +1047,7 @@ size_t _starpu_memory_reclaim_generic(unsigned node, unsigned force, size_t recl
 {
 	size_t freed = 0;
 
+	STARPU_ASSERT(node < STARPU_MAXNODES);
 	if (reclaim && !force)
 	{
 		static unsigned warned;
@@ -999,7 +1057,7 @@ size_t _starpu_memory_reclaim_generic(unsigned node, unsigned force, size_t recl
 			{
 				char name[32];
 				starpu_memory_node_get_name(node, name, sizeof(name));
-				_STARPU_DISP("Not enough memory left on node %s. Your application data set seems too huge to fit on the device, StarPU will cope by trying to purge %lu MiB out. This message will not be printed again for further purges\n", name, (unsigned long) (reclaim / 1048576));
+				_STARPU_DISP("Not enough memory left on node %s. Your application data set seems too huge to fit on the device, StarPU will cope by trying to purge %lu MiB out. This message will not be printed again for further purges\n", name, (unsigned long) ((reclaim+1048575) / 1048576));
 			}
 		}
 	}
@@ -1032,6 +1090,7 @@ void starpu_memchunk_tidy(unsigned node)
 	starpu_ssize_t available;
 	size_t target, amount;
 
+	STARPU_ASSERT(node < STARPU_MAXNODES);
 	if (!can_evict(node))
 		return;
 
@@ -1220,7 +1279,7 @@ void starpu_memchunk_tidy(unsigned node)
 		{
 			char name[32];
 			starpu_memory_node_get_name(node, name, sizeof(name));
-			_STARPU_DISP("Low memory left on node %s (%ldMiB over %luMiB). Your application data set seems too huge to fit on the device, StarPU will cope by trying to purge %lu MiB out. This message will not be printed again for further purges. The thresholds can be tuned using the STARPU_MINIMUM_AVAILABLE_MEM and STARPU_TARGET_AVAILABLE_MEM environment variables.\n", name, (long) (available / 1048576), (unsigned long) (total / 1048576), (unsigned long) (amount / 1048576));
+			_STARPU_DISP("Low memory left on node %s (%ldMiB over %luMiB). Your application data set seems too huge to fit on the device, StarPU will cope by trying to purge %lu MiB out. This message will not be printed again for further purges. The thresholds can be tuned using the STARPU_MINIMUM_AVAILABLE_MEM and STARPU_TARGET_AVAILABLE_MEM environment variables.\n", name, (long) (available / 1048576), (unsigned long) (total / 1048576), (unsigned long) ((amount+1048575) / 1048576));
 		}
 	}
 
@@ -1290,6 +1349,7 @@ void _starpu_request_mem_chunk_removal(starpu_data_handle_t handle, struct _star
 
 	STARPU_ASSERT(mc->data == handle);
 	_starpu_spin_checklocked(&handle->header_lock);
+	STARPU_ASSERT(node < STARPU_MAXNODES);
 
 	/* Record the allocated size, so that later in memory
 	 * reclaiming we can estimate how much memory we free
@@ -1386,15 +1446,20 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 	/* perhaps we can directly reuse a buffer in the free-list */
 	uint32_t footprint = _starpu_compute_data_footprint(handle);
 
+	int prefetch_oom = is_prefetch && prefetch_out_of_memory[dst_node];
+
 #ifdef STARPU_USE_ALLOCATION_CACHE
-       _STARPU_TRACE_START_ALLOC_REUSE(dst_node, data_size, handle, is_prefetch);
+	if (!prefetch_oom)
+		_STARPU_TRACE_START_ALLOC_REUSE(dst_node, data_size, handle, is_prefetch);
 	if (try_to_find_reusable_mc(dst_node, handle, replicate, footprint))
 	{
 		_starpu_allocation_cache_hit(dst_node);
-               _STARPU_TRACE_END_ALLOC_REUSE(dst_node, handle, 1);
+		if (!prefetch_oom)
+			_STARPU_TRACE_END_ALLOC_REUSE(dst_node, handle, 1);
 		return data_size;
 	}
-       _STARPU_TRACE_END_ALLOC_REUSE(dst_node, handle, 0);
+	if (!prefetch_oom)
+		_STARPU_TRACE_END_ALLOC_REUSE(dst_node, handle, 0);
 #endif
 	STARPU_ASSERT(handle->ops);
 	STARPU_ASSERT(handle->ops->allocate_data_on_node);
@@ -1415,7 +1480,8 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 
 	do
 	{
-               _STARPU_TRACE_START_ALLOC(dst_node, data_size, handle, is_prefetch);
+		if (!prefetch_oom)
+			_STARPU_TRACE_START_ALLOC(dst_node, data_size, handle, is_prefetch);
 
 #if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 		if (starpu_node_get_kind(dst_node) == STARPU_CUDA_RAM)
@@ -1429,7 +1495,8 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 #endif
 
 		allocated_memory = handle->ops->allocate_data_on_node(data_interface, dst_node);
-               _STARPU_TRACE_END_ALLOC(dst_node, handle, allocated_memory);
+		if (!prefetch_oom)
+			_STARPU_TRACE_END_ALLOC(dst_node, handle, allocated_memory);
 
 		if (allocated_memory == -ENOMEM)
 		{
@@ -1439,9 +1506,11 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 			/* First try to flush data explicitly marked for freeing */
 			size_t freed = flush_memchunk_cache(dst_node, reclaim);
 
-			if (freed >= reclaim)
+			if (freed >= reclaim) {
 				/* That freed enough data, retry allocating */
+				prefetch_out_of_memory[dst_node] = 0;
 				continue;
+			}
 			reclaim -= freed;
 
 			/* Try to reuse an allocated data with the same interface (to avoid spurious free/alloc) */
@@ -1455,8 +1524,13 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 			}
 
 			if (is_prefetch)
+			{
 				/* It's just prefetch, don't bother existing allocations */
+				/* And don't bother tracing allocation attempts */
+				prefetch_out_of_memory[dst_node] = 1;
+				/* TODO: ideally we should not even try to allocate when we know we have not freed anything */
 				continue;
+			}
 
 			if (!told_reclaiming)
 			{
@@ -1468,7 +1542,9 @@ static starpu_ssize_t _starpu_allocate_interface(starpu_data_handle_t handle, st
 			_STARPU_TRACE_START_MEMRECLAIM(dst_node,is_prefetch);
 			_starpu_memory_reclaim_generic(dst_node, 0, reclaim);
 			_STARPU_TRACE_END_MEMRECLAIM(dst_node,is_prefetch);
-		}
+			prefetch_out_of_memory[dst_node] = 0;
+		} else
+			prefetch_out_of_memory[dst_node] = 0;
 	}
 	while((allocated_memory == -ENOMEM) && attempts++ < 2);
 
@@ -1525,6 +1601,7 @@ int _starpu_allocate_memory_on_node(starpu_data_handle_t handle, struct _starpu_
 	starpu_ssize_t allocated_memory;
 
 	unsigned dst_node = replicate->memory_node;
+	STARPU_ASSERT(dst_node < STARPU_MAXNODES);
 
 	STARPU_ASSERT(handle);
 	_starpu_spin_checklocked(&handle->header_lock);
@@ -1565,6 +1642,7 @@ int _starpu_allocate_memory_on_node(starpu_data_handle_t handle, struct _starpu_
 
 unsigned starpu_data_test_if_allocated_on_node(starpu_data_handle_t handle, unsigned memory_node)
 {
+	STARPU_ASSERT(memory_node < STARPU_MAXNODES);
 	return handle->per_node[memory_node].allocated;
 }
 
@@ -1575,6 +1653,10 @@ void _starpu_memchunk_recently_used(struct _starpu_mem_chunk *mc, unsigned node)
 	if (!mc)
 		/* user-allocated memory */
 		return;
+	STARPU_ASSERT(node < STARPU_MAXNODES);
+	if (!can_evict(node))
+		/* Don't bother */
+		return;
 	_starpu_spin_lock(&mc_lock[node]);
 	MC_LIST_ERASE(node, mc);
 	mc->wontuse = 0;
@@ -1589,6 +1671,10 @@ void _starpu_memchunk_wont_use(struct _starpu_mem_chunk *mc, unsigned node)
 	if (!mc)
 		/* user-allocated memory */
 		return;
+	STARPU_ASSERT(node < STARPU_MAXNODES);
+	if (!can_evict(node))
+		/* Don't bother */
+		return;
 	_starpu_spin_lock(&mc_lock[node]);
 	/* Avoid preventing it from being evicted */
 	mc->diduse = 1;
@@ -1613,6 +1699,10 @@ void _starpu_memchunk_dirty(struct _starpu_mem_chunk *mc, unsigned node)
 	if (mc->home)
 		/* Home is always clean */
 		return;
+	STARPU_ASSERT(node < STARPU_MAXNODES);
+	if (!can_evict(node))
+		/* Don't bother */
+		return;
 	_starpu_spin_lock(&mc_lock[node]);
 	if (mc->relaxed_coherency == 1)
 	{
@@ -1637,6 +1727,7 @@ void _starpu_memchunk_dirty(struct _starpu_mem_chunk *mc, unsigned node)
 #ifdef STARPU_MEMORY_STATS
 void _starpu_memory_display_stats_by_node(FILE *stream, int node)
 {
+	STARPU_ASSERT(node < STARPU_MAXNODES);
 	_starpu_spin_lock(&mc_lock[node]);
 
 	if (!_starpu_mem_chunk_list_empty(&mc_list[node]))
@@ -1772,11 +1863,11 @@ choose_target(starpu_data_handle_t handle, unsigned node)
 					break;
 				}
 			}
-		}
-		/* no place in RAM */
-		if (target == -1)
-		{
-			target = get_better_disk_can_accept_size(handle, node);
+			/* no place in RAM */
+			if (target == -1)
+			{
+				target = get_better_disk_can_accept_size(handle, node);
+			}
 		}
 	}
 	/* we haven't the right to write on the disk */
@@ -1786,35 +1877,6 @@ choose_target(starpu_data_handle_t handle, unsigned node)
 	return target;
 }
 
-/* Whether this memory node can evict data to another node */
-/* We use an accelerator -> CPU RAM -> disk storage hierarchy */
-static int
-can_evict(unsigned node)
-{
-	enum starpu_node_kind kind = starpu_node_get_kind(node);
-
-	if (kind == STARPU_DISK_RAM)
-		/* TODO: disk hierarchy */
-		return 0;
-
-	if (kind != STARPU_CPU_RAM)
-		/* This is an accelerator, we can evict to main RAM */
-		return 1;
-
-	/* This is main RAM */
-	unsigned i;
-	unsigned nnodes = starpu_memory_nodes_get_count();
-	for (i = 0; i < nnodes; i++)
-	{
-		if (starpu_node_get_kind(i) == STARPU_DISK_RAM)
-			/* Can evict it to that disk */
-			return 1;
-	}
-
-	/* No disk to push main RAM to */
-	return 0;
-}
-
 void starpu_data_set_user_data(starpu_data_handle_t handle, void* user_data)
 {
 	handle->user_data = user_data;
