@@ -1,9 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013,2016                           Inria
- * Copyright (C) 2013                                     Simon Archipoff
- * Copyright (C) 2008-2014,2016-2017                      Université de Bordeaux
- * Copyright (C) 2010-2015                                CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Simon Archipoff
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,6 +24,7 @@
 #include <core/sched_policy.h>
 #include <sched_policies/fifo_queues.h>
 #include <core/debug.h>
+#include <core/task.h>
 
 static int _random_push_task(struct starpu_task *task, unsigned prio)
 {
@@ -92,13 +92,13 @@ static int random_push_task(struct starpu_task *task)
 
 static void initialize_random_policy(unsigned sched_ctx_id)
 {
-	starpu_sched_ctx_create_worker_collection(sched_ctx_id, STARPU_WORKER_LIST);
+	(void) sched_ctx_id;
 	starpu_srand48(time(NULL));
 }
 
 static void deinitialize_random_policy(unsigned sched_ctx_id)
 {
-	starpu_sched_ctx_delete_worker_collection(sched_ctx_id);
+	(void) sched_ctx_id;
 }
 
 struct starpu_sched_policy _starpu_sched_random_policy =
@@ -113,5 +113,6 @@ struct starpu_sched_policy _starpu_sched_random_policy =
 	.post_exec_hook = NULL,
 	.pop_every_task = NULL,
 	.policy_name = "random",
-	.policy_description = "weighted random based on worker overall performance"
+	.policy_description = "weighted random based on worker overall performance",
+	.worker_type = STARPU_WORKER_LIST,
 };
