@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013,2015                           CNRS
- * Copyright (C) 2010-2011,2013-2016                      Université de Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,12 +26,16 @@
 
 #ifdef STARPU_QUICK_CHECK
 static unsigned ntasks = 64;
+#elif !defined(STARPU_LONG_CHECK)
+static unsigned ntasks = 1000;
 #else
-static unsigned ntasks = 200000;
+static unsigned ntasks = 50000;
 #endif
 
-void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+void dummy_func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet dummy_codelet =
@@ -45,8 +48,7 @@ static struct starpu_codelet dummy_codelet =
 	.nbuffers = 0
 };
 
-
-int main(int argc, char **argv)
+int main(void)
 {
 	double timing;
 	double start;
